
architecture rtl of nb_bfu is
  --BF variables
  alias pos_rd_imm : std_ulogic_vector(3 downto 0) is IMM(3 downto 0);

  signal extr_i : data_word_type;
  signal ins    : data_word_type;
  signal pos_rd : std_ulogic_vector(3 downto 0);

  signal bf_mask   : data_word_type;
  signal ins_mask : data_word_type;
  signal shift_amt : std_ulogic_vector(3 downto 0);
  signal mov_shift : std_ulogic;

  signal shift_extm_level1 : data_word_type;
  signal shift_extm_level2 : data_word_type;
  signal shift_extm_level3 : data_word_type;
  signal shift_extm_level4 : data_word_type;
  signal shift_extm_mov    : data_word_type;

  signal shift_mask_level1 : data_word_type;
  signal shift_mask_level2 : data_word_type;
  signal shift_mask_level3 : data_word_type;
  signal shift_mask_level4 : data_word_type;
  signal shift_mask_mov    : data_word_type;

begin
  
  bf_mask      <= MSB_MASK;
  extr_i       <= EXTR_SHIFT;
  shift_amt    <= pos_rd;
  ins_mask     <= not shift_mask_level4;
  BF_RES       <= (shift_extm_level4 or (ins and ins_mask));

  bf : process (pos_rd_imm, SRC, shift_amt, bf_mask, shift_extm_level1, mov_shift, extr_i,
                shift_extm_level2, shift_extm_level3, shift_mask_level1,
                shift_mask_level2, shift_mask_level3,shift_mask_mov, shift_extm_mov, BFU_SEL)
  begin

    ins    <= x"0000";
    pos_rd <= pos_rd_imm;
    mov_shift <= '0';

    case (BFU_SEL) is
      when sIns   =>
        ins    <= SRC;
      when sMov =>                    
        mov_shift <= '1';
        pos_rd <= SRC(3 downto 0);
        when sMovi =>
      when sDefaultIns =>
        pos_rd <= "0111";
        ins    <= SRC;
      when sDefaultMov =>                    -- sDefaultMov
        pos_rd <= "0111";
      when others =>                    --sMovi
        
    end case;

  if (mov_shift = '1') then
    shift_extm_mov <= ( '0' & extr_i(15 downto 1));
    shift_mask_mov <= ( '0' & bf_mask(15 downto 1));
  else
    shift_extm_mov <= extr_i;
    shift_mask_mov <= bf_mask;
  end if;

  if (shift_amt(3) = '0') then
    -- Shift right by 8
    shift_extm_level1 <= ( x"00" & shift_extm_mov(15 downto 8));
    shift_mask_level1 <= ( x"00" & shift_mask_mov(15 downto 8));
  else
    shift_extm_level1 <= shift_extm_mov;
    shift_mask_level1 <= shift_mask_mov;
  end if;

  if (shift_amt(2) = '0') then
    -- Shift right by 4
    shift_extm_level2 <= ( x"0" & shift_extm_level1(15 downto 4));
    shift_mask_level2 <= ( x"0" & shift_mask_level1(15 downto 4));
  else
    shift_extm_level2 <= shift_extm_level1;
    shift_mask_level2 <= shift_mask_level1;
  end if;

  if (shift_amt(1) = '0') then
    -- Shift right by 2
    shift_extm_level3 <= ( "00" & shift_extm_level2(15 downto 2));
    shift_mask_level3 <= ( "00" & shift_mask_level2(15 downto 2));
  else
    shift_extm_level3 <= shift_extm_level2;
    shift_mask_level3 <= shift_mask_level2;
  end if;

  if (shift_amt(0) = '0') then
    -- Shift right by 1
    shift_extm_level4 <= ( '0' & shift_extm_level3(15 downto 1));
    shift_mask_level4 <= ( '0' & shift_mask_level3(15 downto 1));
  else
    shift_extm_level4 <= shift_extm_level3;
    shift_mask_level4 <= shift_mask_level3;
  end if;

end process;

end rtl;
