
architecture rtl of nb_clz is

  signal input   : data_word_type;
  signal lz      : data_word_type;
  signal stage_1 : std_ulogic_vector(7 downto 0);
  signal stage_2 : std_ulogic_vector(3 downto 0);
  signal stage_3 : std_ulogic_vector(1 downto 0);

  alias carry_w    : std_ulogic is CLZ_SR(0);
  alias zero_w     : std_ulogic is CLZ_SR(1);
  alias negative_w : std_ulogic is CLZ_SR(2);
  alias overflow_w : std_ulogic is CLZ_SR(3);


begin  -- rtl

  input   <= SRC;
  CLZ_RES <= lz;

  carry_w <= '0';
  zero_w <= lz(4);
  negative_w <= input(15);
  overflow_w <= '0';
  
  count_leading_zeroes : process (input, stage_1, stage_2, stage_3)
  begin  -- process count_leading_zeroes

    lz <= x"0000";

    if input = x"0000" then
      stage_1 <= (others => '0');
      stage_2 <= (others => '0');
      stage_3 <= (others => '0');
      lz(4)   <= '1';
    else

      if input(15 downto 8) = x"00" then
        lz(3)   <= '1';
        stage_1 <= input(7 downto 0);
      else
        stage_1 <= input(15 downto 8);
      end if;

      if stage_1(7 downto 4) = x"0" then
        lz(2)   <= '1';
        stage_2 <= stage_1(3 downto 0);
      else
        stage_2 <= stage_1(7 downto 4);
      end if;

      if stage_2(3 downto 2) = "00" then
        lz(1)   <= '1';
        stage_3 <= stage_2(1 downto 0);
      else
        stage_3 <= stage_2(3 downto 2);
      end if;

      if stage_3(1) = '0' then
        lz(0) <= '1';
      end if;

    end if;

  end process count_leading_zeroes;

end rtl;

