--------------------------------------------------------------------------------
--  Shifter RTL Architecture
--  
--  
--------------------------------------------------------------------------------

architecture rtl of nb_shift is 
	signal input 			: data_word_type;
	
	signal SHIFT_RES_int 	: data_word_type;
	
	signal left_carry		: std_ulogic;
	signal right_carry		: std_ulogic;
	
	alias carry_w			: std_ulogic is SHIFT_SR(0);
	alias zero_w			: std_ulogic is SHIFT_SR(1);
	alias negative_w		: std_ulogic is SHIFT_SR(2);
	alias overflow_w		: std_ulogic is SHIFT_SR(3);
	                    	
	alias carry				: std_ulogic is SR(0);
	                    	
	signal logic_en			: std_ulogic;
	signal arith_en			: std_ulogic;
	signal carry_en			: std_ulogic;
	signal rot_en 			: std_ulogic;

        signal str_en : std_ulogic;

       

	signal shift_l_level1	: data_word_type;
	signal shift_l_level2	: data_word_type;
	signal shift_l_level3	: data_word_type;
	signal shift_l_level4	: data_word_type;

        signal shift_s_level1	: data_word_type;
	signal shift_s_level2	: data_word_type;
	signal shift_s_level3	: data_word_type;
	signal shift_s_level4	: data_word_type;
	                           
        signal shift_r_level1	: data_word_type;
	signal shift_r_level2	: data_word_type;
	signal shift_r_level3	: data_word_type;
	signal shift_r_level4	: data_word_type;

	signal shift_amt		: std_ulogic_vector(3 downto 0);

	alias input_sign_bit 	: std_ulogic is input(15);
	alias result_sign_bit	: std_ulogic is SHIFT_RES_int(15);

-- Vector and operation 
function "and" (L: std_ulogic_vector; R: std_ulogic) return std_ulogic_vector is
	variable i 			: integer;
	variable retval 	: std_ulogic_vector(L'range);
begin
	for i in L'range loop
		retval(i) := L(i) and R;
	end loop;
	return retval;
end function;

-- Vector or operation 
function "or" (L: std_ulogic_vector; R: std_ulogic) return std_ulogic_vector is
	variable i 			: integer;
	variable retval 	: std_ulogic_vector(L'range);
begin
	for i in L'range loop
		retval(i) := L(i) or R;
	end loop;
	return retval;
end function;

begin
	-- Assignments	
	SHIFT_RES		<= SHIFT_RES_int;
	
	carry_w 		<= left_carry when (SHIFT_CONTROL.dir = Left) else right_carry;
	zero_w 			<= '1' when (U_UNSIGNED(SHIFT_RES_int) = 0) else '0';
	negative_w		<= SHIFT_RES_int(15);
	overflow_w 		<= input_sign_bit xor result_sign_bit; -- Input sign xor result sign
	
	logic_en 	<= '1' when (SHIFT_CONTROL.shift_in = sL) else '0';
	arith_en	<= '1' when (SHIFT_CONTROL.shift_in = sA) else '0';
	carry_en	<= '1' when ((SHIFT_CONTROL.shift_in = sC) and (carry = '1')) else '0';
	rot_en		<= '1' when (SHIFT_CONTROL.shift_in = sRot) else '0';
        str_en          <= '1' when (SHIFT_CONTROL.shift_in = sStr) else '0';
	
	input 		<= RS2;
	
	shifter : process (RS2, SHIFT_CONTROL, arith_en, carry_en, input, rot_en, str_en, shift_amt, 
                     shift_l_level1, shift_l_level2, shift_l_level3, RS,
                     shift_l_level4, shift_r_level1, shift_r_level2,
                     shift_r_level3, shift_r_level4, input_sign_bit)
	begin

                case (SHIFT_CONTROL.amt_ctrl) is
                  when sRs =>  shift_amt <= RS(3 downto 0);
                  --when sStr => shift_amt <= STREAM_COUNT(3 downto 0);
                  when others => shift_amt <= SHIFT_CONTROL.amt;
                end case;
          
		
		----------------------------------------------------------------------
		-- Shift Left
		----------------------------------------------------------------------
		if (shift_amt(3) = '1') then 
			-- Shift by 8
                        shift_s_level1  <= STREAM(7 downto 0) & x"00";
			shift_l_level1	<= input(7 downto 0) & (((input(15 downto 8) and rot_en) or (STREAM(15 downto 8) and str_en)) or carry_en);
		else
			-- Pass
                        shift_s_level1	<= STREAM;
			shift_l_level1	<= input;
		end if;
		if (shift_amt(2) = '1') then
			-- Shift by 4
                        shift_s_level2	<= shift_s_level1(11 downto 0) & x"0";
			shift_l_level2	<= shift_l_level1(11 downto 0) & ((shift_l_level1(15 downto 12) and rot_en) or (shift_s_level1(15 downto 12) and str_en) or carry_en);
		else
			-- Pass
                        shift_s_level2	<= shift_s_level1;
			shift_l_level2	<= shift_l_level1;
		end if;
		if (shift_amt(1) = '1') then
			-- Shift by 2
                        shift_s_level3	<= shift_s_level2(13 downto 0) &  "00" ;
			shift_l_level3	<= shift_l_level2(13 downto 0) & ((shift_l_level2(15 downto 14) and rot_en) or (shift_s_level2(15 downto 14) and str_en) or carry_en);
		else
			-- Pass
                  	shift_s_level3	<= shift_s_level2;
			shift_l_level3	<= shift_l_level2;
		end if;
		if (shift_amt(0) = '1') then
			-- Shift by 1
                        shift_s_level4	<= shift_s_level3(14 downto 0) & '0';
			shift_l_level4	<= shift_l_level3(14 downto 0) & ((shift_l_level3(15 downto 15) and rot_en) or (shift_s_level3(15 downto 15) and str_en) or carry_en);
		else
			-- Pass
                  	shift_s_level4	<= shift_s_level3; 
			shift_l_level4	<= shift_l_level3;
		end if;
		
		-- Set carry to last bit shifted out
		if (shift_amt(0) = '1') then
			left_carry <= shift_l_level3(15);
		elsif (shift_amt(1) = '1') then
			left_carry <= shift_l_level2(14);
		elsif (shift_amt(2) = '1') then
			left_carry <= shift_l_level1(12);
		elsif (shift_amt(3) = '1') then
			left_carry <= input(8);
		else
			left_carry <= '0';
		end if;
		

		----------------------------------------------------------------------
		-- Shift Right	
		----------------------------------------------------------------------
		if (shift_amt(3) = '1') then 
			-- Shift by 8
			shift_r_level1	<= ((input(7 downto 0) and rot_en) or (carry_en or (arith_en and input_sign_bit))) & input(15 downto 8);
		else
			-- Pass
			shift_r_level1	<= input;
		end if;
		if (shift_amt(2) = '1') then
			-- Shift by 4
			shift_r_level2	<= ((shift_r_level1(3 downto 0) and rot_en) or (carry_en or (arith_en and input_sign_bit))) & shift_r_level1(15 downto 4);
		else
			-- Pass
			shift_r_level2	<= shift_r_level1;
		end if;
		if (shift_amt(1) = '1') then
			-- Shift by 2
			shift_r_level3	<= ((shift_r_level2(1 downto 0)  and rot_en) or (carry_en or (arith_en and input_sign_bit))) & shift_r_level2(15 downto 2);
		else
			-- Pass
			shift_r_level3	<= shift_r_level2;
		end if;
		if (shift_amt(0) = '1') then
			-- Shift by 1
			shift_r_level4	<= ((shift_r_level3(0 downto 0) and rot_en) or (carry_en or (arith_en and input_sign_bit))) & shift_r_level3(15 downto 1);
		else
			-- Pass
			shift_r_level4	<= shift_r_level3;
		end if;
		
		-- Set carry to last bit shifted out
		if (shift_amt(0) = '1') then
			right_carry <= shift_r_level3(0);
		elsif (shift_amt(1) = '1') then
			right_carry <= shift_r_level2(1);
		elsif (shift_amt(2) = '1') then
			right_carry <= shift_r_level1(3);
		elsif (shift_amt(3) = '1') then
			right_carry <= input(7);
		else
			right_carry <= '0';
		end if;
		
		
		-- Output mux
		if (SHIFT_CONTROL.dir = Left) then
			SHIFT_RES_int <= shift_l_level4;
		else -- Right
			SHIFT_RES_int <= shift_r_level4;
		end if;
			
				
	end process;
	
end rtl;
